/*
 * file: pv_jvisu.c
 * This file is part of the libpv library.
 *
 * Copyright 2005 Pengutronix
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 */

#include <libpv_config.h>

#include <errno.h>
#include <inttypes.h>
#include <string.h>

#include <pv_jvisu.h>
#include <jv_server.h>

/**
 * pv_jvisu_init
 * @engine: Engine to work for
 * @port: Port number to use for JVisu connection
 *
 * Open a network connection and listen to it
 * Doesn't do anything if @port is 0. This means don't listen
 * for a JVisu client.
 *
 * Return value: EBUSY if a connections is already done, ENOMEM ?,
 * IDs from socket open or 0 if ok or nothing done here
 **/
int pv_jvisu_init(struct pv_engine	*engine,
		  uint16_t		port)
{
	struct jv_server	*jv_server;
	int			err, sd;

	if (port == 0)
		return 0;	/* Nothing to do here */

	if (engine->jv_server)
		return -EBUSY;

	jv_server = engine->jv_server = (struct jv_server *)calloc(1, sizeof(struct jv_server));
	if (!engine->jv_server) {
		dprintf(LEVEL_CRIT, "(%s) failed to allocate memory for jv_server\n",
			__PRETTY_FUNCTION__);
		err = -ENOMEM;
		goto exit_0;
	}

	jv_server->pv_engine = engine;

	/* open bind and listen on socket */
	sd = jv_server_start_socket(jv_server, port);
	if (sd < 0) {
		dprintf(LEVEL_CRIT, "(%s) cannot open socket (port: %d)\n",
			__PRETTY_FUNCTION__, port);
		err = -errno;
		goto exit_1;
	}
	dprintf(LEVEL_DEBUG, "(%s) active (sd=%d), waiting for connections on port %d\n",
		__PRETTY_FUNCTION__, sd, port);

	/* preset active SDs with -1 */
	memset(&jv_server->jv_active_sock, 0xff, sizeof (jv_server->jv_active_sock));

	/* store active socket */
	jv_server->jv_active_sock[0].sd = sd;

	FD_ZERO(&jv_server->jv_read_fdset);
	FD_ZERO(&jv_server->jv_write_fdset);
	
	/* server is always set */
	FD_SET(sd, &jv_server->jv_read_fdset);

	err = pthread_create(&jv_server->jvsocket_thread, 0, jv_server_socket_task, jv_server);
	if (err) {
		dprintf(LEVEL_CRIT, "(%s) failed to create jvisu thread (err %d)\n",
			__PRETTY_FUNCTION__, err);
		goto exit_1;
	}

	return 0;

 exit_1:
	free(engine->jv_server);
 exit_0:
	return err;
}

/**
 * pv_jvisu_destroy
 * @engine:  Engine to work for
 *
 * Description
 *
 * Return value:
 **/
int pv_jvisu_destroy(struct pv_engine	*engine)
{
	struct jv_server	*jv_server = engine->jv_server;

	if (!jv_server)		/* no mem alloced, thus not running */
		return 0;

	jv_server->jvisu_stop = 1;

	dprintf(LEVEL_DEBUG, "(%s) waiting for server thread\n", __FUNCTION__);
	pthread_join(jv_server->jvsocket_thread, NULL);
	dprintf(LEVEL_INFO, "(%s) joined jvisu server thread\n", __FUNCTION__);

	return 0;
}

/* end of file pv_jvisu.c */
